/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2005)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: i2cs_genric.h,v 1.5 2005-04-27 19:23:00 zer0 Exp $
 *
 */

/* preliminary definition of i2c module interface */

/** read carefully !

\verbatim

  this implementation of the i2c interface is very specific
  this is a multi bus implementation. The multi bus operation is done by implementing
  software i2c modules (slave only)

  HARDWARE I2C :
  --------------

  the HW i2c has the always master number 0 !
  you can use the hardware i2c module of the avr microcontroller in i2c slave and/or master mode
  in slave mode, you simply receive or send bytes on demand
  in master mode you can send start, stops, and bytes, look for acks ...
  if you use the Master&slave operation (multimaster), your interface looks like two 
  i2c interfaces : a master and a slave! 
  you can then allow multi-master operation very simply. In this case you MUST handle the abort !
  
  SOFTWARE I2C type Bit :
  -----------------------
  
  The software i2c functions simply by putting the right sequence bits on the good ports, and polling the received cars
  This version can function only in master mode.
  The output timings are done with loops, or with the manage() function (selectable)
  perhaps we will add the clock polling for the slave ready waiting
 
  SOFTWARE I2C type interrupt :
  -----------------------------

  M/S software i2c functionning with an interrupt line 
  (project to be implemented one day if someone needs a multi slave, i doubt ;)
  
\endverbatim
*/


#include <avr/io.h>
#include <utils.h>

#include "i2c_config.h"




/** SLAVE STUFF */

#ifndef _ENUM_I2CS_STATE_
#define _ENUM_I2CS_STATE_
typedef enum { I2CS_READY, I2CS_BUSY,
                I2CS_ADRESSED_RECEIVE, I2CS_ADRESSED_SEND,
                I2CS_SENT_BYTE, I2CS_RECEIVED_BYTE,
                I2CS_SENT_BYTE_NO_ACK,
                I2CS_NOT_INIT
               } i2cs_state;
#endif


/** for the moment, only slave 0 makes sense, one day perhaps, with interrupt i2c which will handle multiple slaves 
   set LSB of slave address for recognition of general call
   !!! real adress is (slave_adress >>1 ) */
extern void i2cs_init(u08 slave_address);

/** managing of a spacific i2c interface, not necessary for all */
extern void i2cs_manage(void);


/* registers the global event */
extern void i2cs_register_event(void (*f)(i2cs_state state)  );

extern i2cs_state i2cs_get_state(void);

/** this functions return 0 if ok, 1 if interface not ready.
    be careful ! this func doesn't return the received value, but starts receiving of a byte */
extern u08 i2cs_receive_byte(void);

extern u08 i2cs_get_received_byte(void);


extern u08 i2cs_send_byte(u08 c);
extern u08 i2cs_send_last_byte(u08 c);







